#include <SoftwareSerial.h>
#include <Servo.h>

#define BT_TX_PIN 2 //TX pin of bluetooth module
#define BT_RX_PIN 4 //RX pin of bluetooth module
#define LEFT_WHEEL_PIN 9
#define RIGHT_WHEEL_PIN 5
#define LEFT_ARM_PIN 6
#define RIGHT_ARM_PIN 3
#define CLK_PIN 11
#define CS_PIN 10
#define DIN_PIN 12
#define BUZZER_PIN 8
#define ECO_PIN 7
#define TRIGGER_PIN 13
#define tctr5000_PIN A0 

SoftwareSerial _bt_device(BT_TX_PIN,BT_RX_PIN);
Servo leftWheel;
Servo rightWheel;
Servo leftArm;
Servo rightArm;

int _bt_cmd=0;
int _bt_length=0;
unsigned char _bt_data[255];

#define CMD_SERVO 0x10
#define CMD_SERVO_CONT 0x11
#define CMD_BUZZER_MELODY 0x21
#define CMD_LED_MATRIX_PREDEF_EXPR 0x51

const uint16_t GAME_THRONES[] = {1568,562,1047,562,1245,93,1397,93,1568,375,1047,375,1245,93,1397,93,1175,1875,1397,562,932,562,1245,93,1175,93,1397,375,932,562,1245,93,1175,93,1047,750};
const uint16_t IMPERIAL_MARCH[] = {440,375,440,375,440,375,349,281,523,93,440,375,349,281,523,93,440,750,659,375,659,375,659,375,698,281,523,93,415,375,349,281,523,93,440,750,880,375,440,281,440,93,880,375,831,281,784,93,740,93,698,93,740,187,0,187,446,281,622,375,587,281,554,93,523,93,494,93,523,187,0,187,349,187,415,375,349,281,392,93,523,375,440,281,523,93,659,750,880,375,440,281,440,93,880,375,831,281,784,93,740,93,698,93,740,187,0,187,446,281,622,375,587,281,554,93,523,93,494,93,523,187,0,187,330,187,415,375,330,281,523,93,440,375,349,281,523,93,440,750};
const uint16_t SEVEN_NATIONS_ARMY[] = {330,759,330,252,392,378,330,125,0,252,294,252,262,1012,245,1012,330,759,330,252,392,378,330,125,0,252,294,252,262,506,294,252,262,252,245,1012,330,759,330,252,392,378,330,125,0,252,294,252,262,1012,245,1012};

void LEDMatrix_init(int cs, int din, int clk);
void writeRow(int cs, int din, int clk, int row, int data);
void maxAll (int cs, int din, int clk, int reg, int col);
void putByte (int din, int clk, int data);
void expression(int cs, int din, int clk, int col1, int col2, int col3, int col4, int col5, int col6, int col7, int col8);
void playMelody(int pin,const uint16_t* melody, int length);
void happy();
void sad();
void love();
void angry();
void stunned();
void SigueLineas();
long US_init(int trigger_pin, int echo_pin);
long distance(int trigger_pin, int echo_pin);
void EvitaObstaculos();

void setup()
{
  Serial.begin(9600);
  _bt_device.begin(9600);
  _bt_device.flush();
  String name = "PANDABOT";
  String str = "AT+NAME"+name+"\r\n";
  _bt_device.write(str.c_str());
  pinMode(BUZZER_PIN,OUTPUT);
  pinMode(CLK_PIN,OUTPUT);
  pinMode(CS_PIN,OUTPUT);
  pinMode(DIN_PIN,OUTPUT);
  pinMode(ECO_PIN,INPUT);
  pinMode(TRIGGER_PIN,OUTPUT);
  pinMode(tctr5000_PIN, INPUT);
  
 //Set the LED Matrix component 
  maxAll(CS_PIN,DIN_PIN,CLK_PIN,11,7);
  maxAll(CS_PIN,DIN_PIN,CLK_PIN,9,0);
  maxAll(CS_PIN,DIN_PIN,CLK_PIN,12,1);
  maxAll(CS_PIN,DIN_PIN,CLK_PIN,15,0);
  for (int i = 1; i <= 8; i++)
  maxAll(CS_PIN,DIN_PIN,CLK_PIN,i,0);
  maxAll(CS_PIN,DIN_PIN,CLK_PIN,10,15);
  expression(CS_PIN,DIN_PIN,CLK_PIN,0,0,0,0,0,0,0,0);

  leftWheel.attach(LEFT_WHEEL_PIN);
  delay(300);
  rightWheel.attach(RIGHT_WHEEL_PIN);
  delay(300);
  leftArm.attach(LEFT_ARM_PIN);
  delay(300);
  rightArm.attach(RIGHT_ARM_PIN);
  delay(300);
}

void loop()
{  
  if (_bt_device.available()>0 || _bt_cmd>0)
  {
    int cmd=_bt_device.read();
    if((cmd==1) || (_bt_cmd==1)){
      angry();
    }
    if((cmd==2) || (_bt_cmd==2)){
      sad();
    }
    if((cmd==3) || (_bt_cmd==3)){
      happy();
    }
    if((cmd==4) || (_bt_cmd==4)){
      love();
    }
    if((cmd==5) || (_bt_cmd==5)){
      stunned();
    }
    if((cmd==21) || (_bt_cmd==21)){
      leftWheel.write(((100*90)/100+90));
      rightWheel.write(-((100*90)/100+90));
    }
    if((cmd==22) || (_bt_cmd==22)){
      leftWheel.write(-((100*90)/100+90));
      rightWheel.write(((100*90)/100+90));
    }
    if((cmd==23) || (_bt_cmd==23)){
      _bt_cmd=cmd;
      leftWheel.write(90);
      rightWheel.write(-((100*90)/100+90));
    }
    if((cmd==24) || (_bt_cmd==24)){
      leftWheel.write(((100*90)/100+90));
      rightWheel.write(90);
    }
    if((cmd==25) || (_bt_cmd==25)){
      leftWheel.write(90);
      rightWheel.write(90);
    }
    if((cmd==31) || (_bt_cmd==31)){
      leftArm.write(180);
      delay(1000);
      leftArm.write(0);
      delay(1000);
      leftArm.write(90);
    }
    if((cmd==33) || (_bt_cmd==33)){
      rightArm.write(180);
      delay(1000);
      rightArm.write(0);
      delay(1000);
      rightArm.write(90);
    }
    if((cmd==41) || (_bt_cmd==41)){
      do{
      EvitaObstaculos();}while(_bt_device.read()!=25);
   }
    if((cmd==42) || (_bt_cmd==42)){
      do{
        SigueLineas();
      }while(_bt_device.read()!=25);
    }
  }
}

void LEDMatrix_init(int cs, int din, int clk) {
  maxAll(cs,din,clk,11,7);
  maxAll(cs,din,clk,9,0);
  maxAll(cs,din,clk,12,1);
  maxAll(cs,din,clk,15,0);
  for (int i = 1; i <= 8; i++)
  maxAll(cs,din,clk,i, 0);
  maxAll(cs,din,clk,10,15);
}

void writeRow(int cs, int din, int clk, int row, int data) {
  digitalWrite(cs,LOW);
  putByte(din,clk,row);
  putByte(din,clk,data);
  digitalWrite(cs,LOW);
  digitalWrite(cs,HIGH);
}

void maxAll (int cs, int din, int clk, int reg, int col) {
  digitalWrite(cs,LOW);
  putByte(din,clk,reg);
  putByte(din,clk,col); 
  digitalWrite(cs,LOW);
  digitalWrite(cs,HIGH);
}

void putByte (int din, int clk, int data) {
  byte i = 8;
  byte mask;
  while(i > 0) {mask = 0x01 << (i - 1);
  digitalWrite(clk,LOW);
  if (data & mask)
  {
    digitalWrite(din,HIGH);
  }
  else
  {
    digitalWrite(din,LOW);
  }
  digitalWrite(clk,HIGH);
  --i;
  }
}

void expression(int cs, int din, int clk, int col1, int col2, int col3, int col4, int col5, int col6, int col7, int col8) {
  writeRow(cs,din,clk,1, col1);
  writeRow(cs,din,clk,2, col2);
  writeRow(cs,din,clk,3, col3);
  writeRow(cs,din,clk,4, col4);
  writeRow(cs,din,clk,5, col5);
  writeRow(cs,din,clk,6, col6);
  writeRow(cs,din,clk,7, col7);
  writeRow(cs,din,clk,8, col8);
}

void playMelody(int pin,const uint16_t* melody, int length){
  unsigned int note;
  unsigned long duration;
  uint16_t* melody_ptr=(uint16_t*)melody;
  for (int i=0;i<length;i++)
  {
    note=*melody_ptr++;
    duration=*melody_ptr++;
    tone(pin,note,duration);
    delay(duration);
    noTone(pin);
  }
}

void happy () {
  expression(10,12,11,12,24,48,48,48,48,24,12);
}

void sad () {
  expression(10,12,11,48,24,12,12,12,12,24,48);
}

void love () {
  expression(10,12,11,12,30,62,124,124,62,30,12);
}

void angry () {
  expression(10,12,11,2,68,36,16,16,36,68,2);
}

void stunned () {
  expression(10,12,11,0,0,0,24,24,0,0,0);
}

void SigueLineas(){
  double colour;
  double e, Vm=400, k=50, v=5;
  double V_I,V_D;
  colour=analogRead(tctr5000_PIN);
  e=(Vm-colour)/k;
  V_I=v-e;
  V_D=v+e;
  leftWheel.write(90+V_I);
  rightWheel.write((90-V_D));
}

long US_init(int trigger_pin, int echo_pin)
{
  digitalWrite(trigger_pin, LOW);
  delayMicroseconds(2);
  digitalWrite(trigger_pin, HIGH);
  delayMicroseconds(10);
  digitalWrite(trigger_pin, LOW);
  long microseconds = pulseIn(echo_pin ,HIGH);
  return microseconds;
}

long distance(int trigger_pin, int echo_pin)
{
  long microseconds = US_init(trigger_pin, echo_pin);
  long distance;
  distance = microseconds/29/2;
  if (distance == 0){
  distance = 999;
  }
  return distance;
}

void EvitaObstaculos(){
  long d;
  double dmin=35;
  d=distance(TRIGGER_PIN, ECO_PIN);
  if(d<dmin){
    leftWheel.write(90);
    rightWheel.write(90);
    delay(100);
    leftWheel.write(70);
    rightWheel.write(0);
    delay(350);
    leftWheel.write(90);
    rightWheel.write(90);
    delay(100);
  }
  else{
    leftWheel.write(150);
    rightWheel.write(0);
    delay(50);
  }
}
